@@ -1,3 +1,5 @@
+r_seed(S) :-
+    r_eval('set.seed'(S)).
+
 r_norm(N, L) :-
-    r_eval('set.seed'(123)),
     r_eval(rnorm(N), L).
